;;;; Patch 4.11: condition friendly OID

(in-package :asn.1)

(setf *minor-version* 11)

(define-condition unknown-object-id (asn-error)
  ((symbol :type symbol
           :reader unknown-object-id-symbol
           :initarg :symbol))
  (:report (lambda (c stream)
             (format stream "Unknown object id: ~A"
                     (unknown-object-id-symbol c)))))

(defun oid-symbol-value (symbol)
  (if (and (boundp symbol)
           (typep (symbol-value symbol) 'object-id))
    (symbol-value symbol)
    (let ((possible (sort (delete-if-not #'(lambda (x) (and (boundp x)
                                                            (typep (symbol-value x)
                                                                   'object-id)))
                                         (apropos-list (symbol-name symbol) :asn.1))
                          #'(lambda (a b) (oid->= (symbol-value a)
                                                  (symbol-value b))))))
      (restart-case
          (error 'unknown-object-id :symbol symbol)
        (new-value ()
          :report (lambda (stream)
                    (format stream "Use possible value ~A" (car possible)))
          (symbol-value (car possible)))))))
