;;;; Patch 4.6: BER-decode unknown data into RAW object but NIL.

(in-package :asn.1)

(defparameter *version* 4.6)

(defun ber-decode-type (stream)
  "Decode BER Type Domain"
  (declare (type stream stream))
  (let ((byte (read-byte stream))
        (type-length 1))
    (let ((class (ldb (byte 2 6) byte))
          (p/c (ldb (byte 1 5) byte))
          (tags (ldb (byte 5 0) byte))
          (bytes (list byte)))
      (when (= tags 31)
        (setf tags (labels ((iter (acc)
                              (setf byte (read-byte stream))
                              (push byte bytes)
                              (incf type-length)
                              (let ((temp (logior (ash acc 7)
						  (ldb (byte 7 0) byte))))
                                (if (= (ldb (byte 1 7) byte) 1)
				    (iter temp)
				    temp))))
                     (iter 0))))
      (values (get-asn.1-type class p/c tags)
              type-length
              (nreverse bytes)))))

(defun ber-decode-length (stream)
  "Decode BER Length Domain"
  (declare (type stream stream))
  (let ((byte (read-byte stream))
        (length-length 1))
    (let ((flag (ldb (byte 1 7) byte))
          (l-or-n (ldb (byte 7 0) byte))
          (bytes (list byte)))
      (let ((length (if (zerop flag) l-or-n
                      (let ((acc 0))
                        (dotimes (i l-or-n acc)
                          (setf byte (read-byte stream)
                                acc (logior (ash acc 8) byte))
                          (push byte bytes)
                          (incf length-length))))))
        (values length
                length-length
                (nreverse bytes))))))

(defmethod ber-decode ((stream stream))
  (multiple-value-bind (type type-bytes) (ber-decode-type stream)
    (multiple-value-bind (length length-bytes) (ber-decode-length stream)
      (if type
        (ber-decode-value stream type length)
        ;; When unknown type detected, recover whole data to a ASN.1 RAW object
        (ber-decode-value stream type (cons length
                                            (append type-bytes length-bytes)))))))

(defmethod ber-decode-value ((stream stream) (type (eql nil)) length)
  "Note: A UNKNOWN BER data will be decoded as NIL"
  (declare (type list length)
           (ignore type))
  (let (value)
    (raw (append (cdr length) ; type and length
                 (dotimes (i (car length) (nreverse value))
                   (push (read-byte stream) value))))))

(defmethod print-object ((object raw) stream)
  (print-unreadable-object (object stream :type t)
    (format stream "~A" (data-of object))))
