;;;; Patch 4-7: PRINT-OBJECT change on OBJECT-ID

(in-package :asn.1)

(defparameter *version* 4.7)

(defun oid-find-leaf (oid)
  "Find the leaf node in a oid's all parents"
  (declare (type object-id oid))
  (unless (oid-trunk-p oid)
    (labels ((iter (o acc)
               (if (oid-leaf-p o)
                 (values o acc)
                 (let ((p (oid-parent o))
                       (v (oid-value o)))
                   (iter p (cons v acc))))))
      (iter oid nil))))

(export 'oid-find-leaf)

(defmethod print-object ((object object-id) stream)
  (print-unreadable-object (object stream :type t)
    (cond ((and (slot-boundp object 'name)
                (slot-boundp object 'value))
           (if (slot-boundp object 'module)
             (format stream "~A::" (oid-module object)))
           (format stream "~A (~D) [~D]"
                   (oid-name object)
                   (oid-value object)
                   (hash-table-count (oid-children object))))
          ((slot-boundp object 'value)
           (multiple-value-bind (base-oid others) (oid-find-leaf object)
             (if (slot-boundp base-oid 'module)
               (format stream "~A::" (oid-module base-oid)))
             (format stream "~A~{.~A~}" (oid-name base-oid) others)))
          (t (format stream ".")))))

;;; remote old OID-SHORT-NAME-LIST which only be used by PRINT-OBJECT
(fmakunbound 'oid-short-name-list)
(unintern 'oid-short-name-list)

(defun make-mib-name-map ()
  (clrhash *mib-name-map-table*)
  (mapcar #'(lambda (x)
              (setf (gethash (car x) *mib-name-map-table*) (cdr x)))
          *mib-name-map*))
