;;;; -*- Mode: Lisp -*-
;;;; $Id: oid-reader.lisp 555 2008-09-28 15:36:14Z binghe $

(in-package :asn.1)

;; sysDescr.0 -> |sysDescr| and 0
;; 1.3.6.1.2.1.1.1.0 -> *root-object-id*, 1, 3, 6, 1, 2, 1, 1, 1, 0
(defvar *oid-readtable*)

(defun make-oid-readtable ()
  ;; Create a new readtable
  (setf *oid-readtable* (copy-readtable nil))
  ;; Case-Sensitivity
  (setf (readtable-case *oid-readtable*) :preserve)
  ;; . reader
  (set-macro-character #\. #'(lambda (stream char)
                               (declare (ignore stream char))
                               (values))
                       nil *oid-readtable*)
  (set-macro-character #\} (get-macro-character #\))
                       t *oid-readtable*))

(eval-when (:load-toplevel :execute)
  (make-oid-readtable))

(defgeneric oid-parse (source))

(defmethod oid-parse ((source string))
  (with-input-from-string (s source)
    (oid-parse s)))

(defmethod oid-parse ((source stream))
  (let ((*readtable* *oid-readtable*)
        (*package* (find-package :asn.1)))
    (loop for i = (read source nil nil nil)
          until (null i)
          collect i)))

(defgeneric oid (object)
  (:documentation "Convert * to OBJECT-ID"))

(defmethod oid ((oid-object object-id))
  oid-object)

(defmethod oid ((oid-vector vector))
  (oid (coerce oid-vector 'list)))

(defmethod oid ((oid-tokens list))
  (labels ((iter (tokens base)
             (if (null tokens)
                 base
               (let ((i (car tokens))
                     (b (gethash (car tokens)
                                 (oid-children base))))
                 (etypecase i
                   (integer (iter (cdr tokens)
                                  (if b b
                                    (make-instance 'object-id
                                                   :value i
                                                   :parent base))))
                   (symbol (iter (cdr tokens)
                                 (oid-symbol-value i))))))))
    (etypecase (car oid-tokens)
      (integer (iter oid-tokens *root-object-id*))
      (symbol (iter (cdr oid-tokens)
                    (oid-symbol-value (car oid-tokens))))
      (object-id (iter (cdr oid-tokens)
                       (car oid-tokens))))))

(defmethod oid ((oid-string string))
  (oid (oid-parse oid-string)))

(defun oid-symbol-value (symbol)
  (if (and (boundp symbol)
           (typep (symbol-value symbol) 'object-id))
    (symbol-value symbol)
    (let ((possible (sort (delete-if-not #'(lambda (x) (and (boundp x)
                                                            (typep (symbol-value x)
                                                                   'object-id)))
                                         (apropos-list (symbol-name symbol) :asn.1))
                          #'(lambda (a b) (oid->= (symbol-value a)
                                                  (symbol-value b))))))
      (restart-case
          (error 'unknown-object-id :symbol symbol)
        (new-value ()
          :report (lambda (stream)
                    (format stream "Use possible value ~A" (car possible)))
          (symbol-value (car possible)))))))
